@@ -1,24 +1,27 @@
-from __future__ import print_function, division, absolute_import
+from __future__ import absolute_import, division, print_function, unicode_literals
 
 import argparse
 import contextlib
+from functools import partial
+import json
 import os
+from os.path import abspath, basename, isfile, join
 import re
 import sys
-from os.path import abspath, basename
 
 from .. import console
-from ..base.constants import ROOT_ENV_NAME, NULL
-from ..base.context import context
-from ..exceptions import (DryRunExit, CondaSystemExit, CondaRuntimeError,
-                          CondaValueError, CondaFileIOError)
-from ..install import dist2quad
+from .._vendor.auxlib.entity import EntityEncoder
+from ..base.constants import ROOT_ENV_NAME, CONDA_TARBALL_EXTENSION
+from ..base.context import context, get_prefix as context_get_prefix
+from ..common.compat import iteritems
+from ..common.constants import NULL
+from ..common.path import is_private_env, prefix_to_env_name
+from ..core.linked_data import linked_data
+from ..exceptions import CondaFileIOError, CondaSystemExit, CondaValueError, DryRunExit
 from ..resolve import MatchSpec
 from ..utils import memoize
 
-# for conda-build 1.21.11 compatibility only
-from conda.base.context import get_prefix as context_get_prefix
-from functools import partial
+
 get_prefix = partial(context_get_prefix, context)
 
 
@@ -85,7 +88,7 @@ class Packages(Completer):
 
     def _get_items(self):
         # TODO: Include .tar.bz2 files for local installs.
-        from ..api import get_index
+        from ..core.index import get_index
         args = self.parsed_args
         call_dict = dict(channel_urls=args.channel or (),
                          use_cache=True,
@@ -94,7 +97,7 @@ class Packages(Completer):
         if hasattr(args, 'platform'):  # in search
             call_dict['platform'] = args.platform
         index = get_index(**call_dict)
-        return [dist2quad(i)[0] for i in index]
+        return [record.name for record in index]
 
 class InstalledPackages(Completer):
     def __init__(self, prefix, parsed_args, **kwargs):
@@ -103,9 +106,9 @@ class InstalledPackages(Completer):
 
     @memoize
     def _get_items(self):
-        import conda.install
-        packages = conda.install.linked(context.prefix_w_legacy_search)
-        return [dist2quad(i)[0] for i in packages]
+        from ..core.linked_data import linked
+        packages = linked(context.prefix_w_legacy_search)
+        return [dist.quad[0] for dist in packages]
 
 def add_parser_help(p):
     """
@@ -167,7 +170,7 @@ def add_parser_json(p):
     p.add_argument(
         "--verbose", "-v",
         action=NullCountAction,
-        help="Use once for info, twice for debug.",
+        help="Use once for info, twice for debug, three times for trace.",
         dest="verbosity",
         default=NULL,
     )
@@ -184,6 +187,8 @@ def add_parser_quiet(p):
 def add_parser_channels(p):
     p.add_argument(
         '-c', '--channel',
+        dest='channel',  # apparently conda-build uses this; someday rename to channels are remove context.channels alias to channel  # NOQA
+        # TODO: if you ever change 'channel' to 'channels', make sure you modify the context.channels property accordingly # NOQA
         action="append",
         help="""Additional channel to search for packages. These are URLs searched in the order
         they are given (including file:// for local directories).  Then, the defaults
@@ -247,6 +252,7 @@ def add_parser_install(p):
     p.add_argument(
         '-f', "--force",
         action="store_true",
+        default=NULL,
         help="Force install (even when package already installed), "
                "implies --no-deps.",
     )
@@ -279,6 +285,8 @@ def add_parser_install(p):
     add_parser_prefix(p)
     add_parser_quiet(p)
     add_parser_copy(p)
+    add_parser_create_install_update(p)
+    add_parser_insecure(p)
     p.add_argument(
         "--alt-hint",
         action="store_true",
@@ -289,7 +297,7 @@ def add_parser_install(p):
         action="store_true",
         dest="update_deps",
         default=NULL,
-        help="Don't update dependencies (default: %s)." % context.update_dependencies,
+        help="Update dependencies (default: %s)." % context.update_dependencies,
     )
     p.add_argument(
         "--no-update-dependencies", "--no-update-deps",
@@ -381,14 +389,31 @@ def add_parser_show_channel_urls(p):
         help="Don't show channel urls.",
     )
 
+
+def add_parser_create_install_update(p):
+    p.add_argument(
+        "--clobber",
+        action="store_true",
+        default=NULL,
+        help="Allow clobbering of overlapping file paths within packages, "
+             "and suppress related warnings.",
+    )
+
+
+def add_parser_insecure(p):
+    p.add_argument(
+        "-k", "--insecure",
+        action="store_false",
+        default=NULL,
+        help="Allow conda to perform \"insecure\" SSL connections and transfers."
+             "Equivalent to setting 'ssl_verify' to 'false'."
+    )
+
+
 def ensure_use_local(args):
     if not args.use_local:
         return
-    try:
-        from conda_build.config import croot  # noqa
-    except ImportError as e:
-        raise CondaRuntimeError("%s: you need to have 'conda-build >= 1.7.1' installed"
-                                " to use the --use-local option." % e)
+
 
 def ensure_override_channels_requires_channel(args, dashc=True):
     if args.override_channels and not (args.channel or args.use_local):
@@ -399,6 +424,7 @@ def ensure_override_channels_requires_channel(args, dashc=True):
             raise CondaValueError('--override-channels requires --channel'
                                   'or --use-local')
 
+
 def confirm(args, message="Proceed", choices=('yes', 'no'), default='yes'):
     assert default in choices, default
     if args.dry_run:
@@ -454,7 +480,7 @@ def ensure_name_or_prefix(args, command):
 
 
 def name_prefix(prefix):
-    if abspath(prefix) == context.root_dir:
+    if abspath(prefix) == context.root_prefix:
         return ROOT_ENV_NAME
     return basename(prefix)
 
@@ -463,7 +489,12 @@ def name_prefix(prefix):
 
 def arg2spec(arg, json=False, update=False):
     try:
-        spec = MatchSpec(spec_from_line(arg), normalize=True)
+        # spec_from_line can return None, especially for the case of a .tar.bz2 extension and
+        #   a space in the path
+        _arg = spec_from_line(arg)
+        if _arg is None and arg.endswith(CONDA_TARBALL_EXTENSION):
+            _arg = arg
+        spec = MatchSpec(_arg, normalize=True)
     except:
         raise CondaValueError('invalid package specification: %s' % arg)
 
@@ -550,7 +581,7 @@ def disp_features(features):
 def stdout_json(d):
     import json
 
-    json.dump(d, sys.stdout, indent=2, sort_keys=True)
+    json.dump(d, sys.stdout, indent=2, sort_keys=True, cls=EntityEncoder)
     sys.stdout.write('\n')
 
 
@@ -559,7 +590,7 @@ def get_index_trap(*args, **kwargs):
     Retrieves the package index, but traps exceptions and reports them as
     JSON if necessary.
     """
-    from ..api import get_index
+    from ..core.index import get_index
     kwargs.pop('json', None)
     return get_index(*args, **kwargs)
 
@@ -575,15 +606,21 @@ def json_progress_bars(json=False):
 
 def stdout_json_success(success=True, **kwargs):
     result = {'success': success}
+
+    # this code reverts json output for plan back to previous behavior
+    #   relied on by Anaconda Navigator and nb_conda
+    actions = kwargs.get('actions', {})
+    if 'LINK' in actions:
+        actions['LINK'] = [str(d) for d in actions['LINK']]
+    if 'UNLINK' in actions:
+        actions['UNLINK'] = [str(d) for d in actions['UNLINK']]
+
     result.update(kwargs)
     stdout_json(result)
 
 
-root_no_rm = 'python', 'pycosat', 'pyyaml', 'conda', 'openssl', 'requests'
-
-
 def handle_envs_list(acc, output=True):
-    from conda import misc
+    from .. import misc
 
     if output:
         print("# conda environments:")
@@ -592,15 +629,62 @@ def handle_envs_list(acc, output=True):
     def disp_env(prefix):
         fmt = '%-20s  %s  %s'
         default = '*' if prefix == context.default_prefix else ' '
-        name = (ROOT_ENV_NAME if prefix == context.root_dir else
+        name = (ROOT_ENV_NAME if prefix == context.root_prefix else
                 basename(prefix))
         if output:
             print(fmt % (name, default, prefix))
 
     for prefix in misc.list_prefixes():
         disp_env(prefix)
-        if prefix != context.root_dir:
+        if prefix != context.root_prefix:
             acc.append(prefix)
 
     if output:
         print()
+
+
+def get_private_envs_json():
+    path_to_private_envs = join(context.root_prefix, "conda-meta", "private_envs")
+    if not isfile(path_to_private_envs):
+        return None
+    try:
+        with open(path_to_private_envs, "r") as f:
+            private_envs_json = json.load(f)
+    except json.decoder.JSONDecodeError:
+        private_envs_json = {}
+    return private_envs_json
+
+
+def prefix_if_in_private_env(spec):
+    private_envs_json = get_private_envs_json()
+    if not private_envs_json:
+        return None
+    prefixes = tuple(prefix for pkg, prefix in iteritems(private_envs_json) if
+                     pkg.startswith(spec))
+    prefix = prefixes[0] if len(prefixes) > 0 else None
+    return prefix
+
+
+def pkg_if_in_private_env(spec):
+    private_envs_json = get_private_envs_json()
+    pkgs = tuple(pkg for pkg, prefix in iteritems(private_envs_json) if pkg.startswith(spec))
+    pkg = pkgs[0] if len(pkgs) > 0 else None
+    return pkg
+
+
+def create_prefix_spec_map_with_deps(r, specs, default_prefix):
+    prefix_spec_map = {}
+    for spec in specs:
+        spec_prefix = prefix_if_in_private_env(spec)
+        spec_prefix = spec_prefix if spec_prefix is not None else default_prefix
+        if spec_prefix in prefix_spec_map.keys():
+            prefix_spec_map[spec_prefix].add(spec)
+        else:
+            prefix_spec_map[spec_prefix] = {spec}
+
+        if is_private_env(prefix_to_env_name(spec_prefix, context.root_prefix)):
+            linked = linked_data(spec_prefix)
+            for linked_spec in linked:
+                if not linked_spec.name.startswith(spec) and r.depends_on(spec, linked_spec):
+                    prefix_spec_map[spec_prefix].add(linked_spec.name)
+    return prefix_spec_map
