@@ -1,4 +1,4 @@
-from __future__ import absolute_import, division, print_function
+from __future__ import absolute_import, division, print_function, unicode_literals
 
 from os.path import join
 
@@ -6,6 +6,7 @@ from .common import name_prefix
 from ..base.context import context
 from ..exceptions import CondaIOError
 
+
 def read_message(fn):
     res = []
     for envs_dir in context.envs_dirs:
@@ -34,9 +35,9 @@ Missing write permissions in: ${root_dir}
 # then make changes to it.
 # This may be done using the command:
 #
-# $ conda create -n my_${name} --clone=${prefix}
+# $ conda create -n my_${name} --clone="${prefix}"
 """
-    msg = msg.replace('${root_dir}', context.root_dir)
+    msg = msg.replace('${root_dir}', context.root_prefix)
     msg = msg.replace('${prefix}', prefix)
     msg = msg.replace('${name}', name_prefix(prefix))
     msg = msg.replace('${command}', command)
