@@ -4,57 +4,34 @@
 # conda is distributed under the terms of the BSD 3-clause license.
 # Consult LICENSE.txt or http://opensource.org/licenses/BSD-3-Clause.
 
-from __future__ import absolute_import, division, print_function
+from __future__ import absolute_import, division, print_function, unicode_literals
 
+from difflib import get_close_matches
 import errno
 import logging
 import os
-import shutil
-import tarfile
-import tempfile
-from difflib import get_close_matches
 from os.path import abspath, basename, exists, isdir, join
+import re
 
-from .. import text_type
+from . import common
 from .._vendor.auxlib.ish import dals
-from ..api import get_index
 from ..base.constants import ROOT_ENV_NAME
-from ..base.context import check_write, context
-from ..cli import common
-from ..cli.find_commands import find_executable
-from ..exceptions import (CondaAssertionError, CondaEnvironmentNotFoundError,
-                          CondaFileNotFoundError, CondaIOError, CondaImportError, CondaOSError,
-                          CondaRuntimeError, CondaSystemExit, CondaValueError,
-                          DirectoryNotFoundError, DryRunExit, LockError, NoPackagesFoundError,
-                          PackageNotFoundError, TooManyArgumentsError, UnsatisfiableError)
-from ..install import is_linked, linked as install_linked, name_dist
+from ..base.context import context
+from ..common.compat import on_win, text_type
+from ..core.index import get_index
+from ..core.linked_data import linked as install_linked
+from ..exceptions import (CondaEnvironmentNotFoundError, CondaIOError, CondaImportError,
+                          CondaOSError, CondaSystemExit, CondaValueError, DirectoryNotFoundError,
+                          DryRunExit, NoPackagesFoundError, PackageNotFoundError,
+                          PackageNotInstalledError, TooManyArgumentsError, UnsatisfiableError)
 from ..misc import append_env, clone_env, explicit, touch_nonadmin
-from ..plan import (add_defaults_to_specs, display_actions, execute_actions, get_pinned_specs,
-                    install_actions, is_root_prefix, nothing_to_do, revert_actions)
-from ..resolve import Resolve
-from ..utils import on_win
+from ..models.channel import prioritize_channels
+from ..plan import (display_actions, execute_actions, get_pinned_specs, install_actions_list,
+                    is_root_prefix, nothing_to_do, revert_actions)
 
 log = logging.getLogger(__name__)
 
 
-def install_tar(prefix, tar_path, verbose=False):
-    if not exists(tar_path):
-        raise CondaFileNotFoundError(tar_path)
-    tmp_dir = tempfile.mkdtemp()
-    t = tarfile.open(tar_path, 'r')
-    t.extractall(path=tmp_dir)
-    t.close()
-
-    paths = []
-    for root, dirs, files in os.walk(tmp_dir):
-        for fn in files:
-            if fn.endswith('.tar.bz2'):
-                paths.append(join(root, fn))
-
-    explicit(paths, prefix, verbose=verbose)
-    shutil.rmtree(tmp_dir)
-
-
 def check_prefix(prefix, json=False):
     name = basename(prefix)
     error = None
@@ -75,7 +52,7 @@ def clone(src_arg, dst_prefix, json=False, quiet=False, index_args=None):
     if os.sep in src_arg:
         src_prefix = abspath(src_arg)
         if not isdir(src_prefix):
-            raise DirectoryNotFoundError(src_arg, 'no such directory: %s' % src_arg, json)
+            raise DirectoryNotFoundError(src_arg)
     else:
         src_prefix = context.clone_src
 
@@ -112,15 +89,27 @@ def print_activate(arg):
         #
         """)
     else:
-        message = dals("""
-        #
-        # To activate this environment, use:
-        # > source activate %s
-        #
-        # To deactivate this environment, use:
-        # > source deactivate %s
-        #
-        """)
+        shell = os.path.split(os.environ.get('SHELL', ''))[-1]
+        if 'fish' == shell:
+            message = dals("""
+            #
+            # To activate this environment, use:
+            # > conda activate %s
+            #
+            # To deactivate this environment, use:
+            # > conda deactivate %s
+            #
+            """)
+        else:
+            message = dals("""
+            #
+            # To activate this environment, use:
+            # > source activate %s
+            #
+            # To deactivate this environment, use:
+            # > source deactivate %s
+            #
+            """)
 
     return message % (arg, arg)
 
@@ -136,7 +125,7 @@ def install(args, parser, command='install'):
     """
     conda install, conda update, and conda create
     """
-    context.validate_all()
+    context.validate_configuration()
     newenv = bool(command == 'create')
     isupdate = bool(command == 'update')
     isinstall = bool(command == 'install')
@@ -154,22 +143,23 @@ def install(args, parser, command='install'):
 # $ conda update --prefix %s anaconda
 """ % prefix)
 
-    linked = install_linked(prefix)
-    lnames = {name_dist(d) for d in linked}
+    args_packages = [s.strip('"\'') for s in args.packages]
+
+    linked_dists = install_linked(prefix)
+    linked_names = tuple(ld.quad[0] for ld in linked_dists)
     if isupdate and not args.all:
-        for name in args.packages:
+        for name in args_packages:
             common.arg2spec(name, json=context.json, update=True)
-            if name not in lnames:
-                raise PackageNotFoundError(name, "Package '%s' is not installed in %s" %
-                                           (name, prefix))
+            if name not in linked_names and common.prefix_if_in_private_env(name) is None:
+                raise PackageNotInstalledError(prefix, name)
 
     if newenv and not args.no_default_packages:
         default_packages = list(context.create_default_packages)
         # Override defaults if they are specified at the command line
         for default_pkg in context.create_default_packages:
-            if any(pkg.split('=')[0] == default_pkg for pkg in args.packages):
+            if any(pkg.split('=')[0] == default_pkg for pkg in args_packages):
                 default_packages.remove(default_pkg)
-        args.packages.extend(default_packages)
+                args_packages.extend(default_packages)
     else:
         default_packages = []
 
@@ -177,7 +167,7 @@ def install(args, parser, command='install'):
     common.ensure_override_channels_requires_channel(args)
     index_args = {
         'use_cache': args.use_index_cache,
-        'channel_urls': args.channel or (),
+        'channel_urls': context.channels,
         'unknown': args.unknown,
         'prepend': not args.override_channels,
         'use_local': args.use_local
@@ -191,36 +181,29 @@ def install(args, parser, command='install'):
             explicit(specs, prefix, verbose=not context.quiet, index_args=index_args)
             return
     elif getattr(args, 'all', False):
-        if not linked:
-            raise PackageNotFoundError('', "There are no packages installed in the "
-                                       "prefix %s" % prefix)
-        specs.extend(nm for nm in lnames)
-    specs.extend(common.specs_from_args(args.packages, json=context.json))
+        if not linked_dists:
+            log.info("There are no packages installed in prefix %s", prefix)
+            return
+        specs.extend(d.quad[0] for d in linked_dists)
+    specs.extend(common.specs_from_args(args_packages, json=context.json))
 
     if isinstall and args.revision:
         get_revision(args.revision, json=context.json)
-    elif isinstall and not (args.file or args.packages):
+    elif isinstall and not (args.file or args_packages):
         raise CondaValueError("too few arguments, "
                               "must supply command line package specs or --file")
 
-    num_cp = sum(s.endswith('.tar.bz2') for s in args.packages)
+    num_cp = sum(s.endswith('.tar.bz2') for s in args_packages)
     if num_cp:
-        if num_cp == len(args.packages):
-            explicit(args.packages, prefix, verbose=not context.quiet)
+        if num_cp == len(args_packages):
+            explicit(args_packages, prefix, verbose=not context.quiet)
             return
         else:
             raise CondaValueError("cannot mix specifications with conda package"
                                   " filenames")
 
-    # handle tar file containing conda packages
-    if len(args.packages) == 1:
-        tar_path = args.packages[0]
-        if tar_path.endswith('.tar'):
-            install_tar(prefix, tar_path, verbose=not context.quiet)
-            return
-
     if newenv and args.clone:
-        package_diff = set(args.packages) - set(default_packages)
+        package_diff = set(args_packages) - set(default_packages)
         if package_diff:
             raise TooManyArgumentsError(0, len(package_diff), list(package_diff),
                                         'did not expect any arguments for --clone')
@@ -228,55 +211,16 @@ def install(args, parser, command='install'):
         clone(args.clone, prefix, json=context.json, quiet=context.quiet, index_args=index_args)
         append_env(prefix)
         touch_nonadmin(prefix)
-        if not context.json:
+        if not context.json and not context.quiet:
             print(print_activate(args.name if args.name else prefix))
         return
 
-    index = get_index(channel_urls=index_args['channel_urls'], prepend=index_args['prepend'],
-                      platform=None, use_local=index_args['use_local'],
-                      use_cache=index_args['use_cache'], unknown=index_args['unknown'],
-                      prefix=prefix)
-    r = Resolve(index)
+    index = get_index(channel_urls=index_args['channel_urls'],
+                      prepend=index_args['prepend'], platform=None,
+                      use_local=index_args['use_local'], use_cache=index_args['use_cache'],
+                      unknown=index_args['unknown'], prefix=prefix)
     ospecs = list(specs)
-    add_defaults_to_specs(r, linked, specs, update=isupdate)
 
-    # Don't update packages that are already up-to-date
-    if isupdate and not (args.all or args.force):
-        orig_packages = args.packages[:]
-        installed_metadata = [is_linked(prefix, dist) for dist in linked]
-        for name in orig_packages:
-            vers_inst = [m['version'] for m in installed_metadata if m['name'] == name]
-            build_inst = [m['build_number'] for m in installed_metadata if m['name'] == name]
-            channel_inst = [m['channel'] for m in installed_metadata if m['name'] == name]
-
-            try:
-                assert len(vers_inst) == 1, name
-                assert len(build_inst) == 1, name
-                assert len(channel_inst) == 1, name
-            except AssertionError as e:
-                raise CondaAssertionError(text_type(e))
-
-            pkgs = sorted(r.get_pkgs(name))
-            if not pkgs:
-                # Shouldn't happen?
-                continue
-            latest = pkgs[-1]
-
-            if all([latest.version == vers_inst[0],
-                    latest.build_number == build_inst[0],
-                    latest.channel == channel_inst[0]]):
-                args.packages.remove(name)
-        if not args.packages:
-            from .main_list import print_packages
-
-            if not context.json:
-                regex = '^(%s)$' % '|'.join(orig_packages)
-                print('# All requested packages already installed.')
-                print_packages(prefix, regex)
-            else:
-                common.stdout_json_success(
-                    message='All requested packages already installed.')
-            return
     if args.force:
         args.no_deps = True
 
@@ -296,16 +240,15 @@ def install(args, parser, command='install'):
 
     try:
         if isinstall and args.revision:
-            actions = revert_actions(prefix, get_revision(args.revision), index)
+            action_set = [revert_actions(prefix, get_revision(args.revision), index)]
         else:
             with common.json_progress_bars(json=context.json and not context.quiet):
-                actions = install_actions(prefix, index, specs,
-                                          force=args.force,
-                                          only_names=only_names,
-                                          pinned=args.pinned,
-                                          always_copy=context.always_copy,
-                                          minimal_hint=args.alt_hint,
-                                          update_deps=context.update_dependencies)
+                _channel_priority_map = prioritize_channels(index_args['channel_urls'])
+                action_set = install_actions_list(
+                    prefix, index, specs, force=args.force, only_names=only_names,
+                    pinned=args.pinned, always_copy=context.always_copy,
+                    minimal_hint=args.alt_hint, update_deps=context.update_dependencies,
+                    channel_priority_map=_channel_priority_map, is_update=isupdate)
     except NoPackagesFoundError as e:
         error_message = [e.args[0]]
 
@@ -342,26 +285,25 @@ def install(args, parser, command='install'):
                     error_message.append("\n\nClose matches found; did you mean one of these?\n")
                 error_message.append("\n    %s: %s" % (pkg, ', '.join(close)))
                 nfound += 1
-            error_message.append('\n\nYou can search for packages on anaconda.org with')
-            error_message.append('\n\n    anaconda search -t conda %s' % pkg)
+            # error_message.append('\n\nYou can search for packages on anaconda.org with')
+            # error_message.append('\n\n    anaconda search -t conda %s' % pkg)
             if len(e.pkgs) > 1:
                 # Note this currently only happens with dependencies not found
                 error_message.append('\n\n(and similarly for the other packages)')
 
-            if not find_executable('anaconda', include_others=False):
-                error_message.append('\n\nYou may need to install the anaconda-client')
-                error_message.append(' command line client with')
-                error_message.append('\n\n    conda install anaconda-client')
+            # if not find_executable('anaconda', include_others=False):
+            #     error_message.append('\n\nYou may need to install the anaconda-client')
+            #     error_message.append(' command line client with')
+            #     error_message.append('\n\n    conda install anaconda-client')
 
             pinned_specs = get_pinned_specs(prefix)
             if pinned_specs:
                 path = join(prefix, 'conda-meta', 'pinned')
                 error_message.append("\n\nNote that you have pinned specs in %s:" % path)
-                error_message.append("\n\n    %r" % pinned_specs)
+                error_message.append("\n\n    %r" % (pinned_specs,))
 
             error_message = ''.join(error_message)
-
-            raise PackageNotFoundError('', error_message)
+            raise PackageNotFoundError(error_message)
 
     except (UnsatisfiableError, SystemExit) as e:
         # Unsatisfiable package specifications/no such revision/import error
@@ -369,63 +311,81 @@ def install(args, parser, command='install'):
             raise CondaImportError(text_type(e))
         raise
 
-    if nothing_to_do(actions) and not newenv:
-        from .main_list import print_packages
-
+    if any(nothing_to_do(actions) for actions in action_set) and not newenv:
         if not context.json:
-            regex = '^(%s)$' % '|'.join(s.split()[0] for s in ospecs)
+            from .main_list import print_packages
+
+            spec_regex = r'^(%s)$' % '|'.join(re.escape(s.split()[0]) for s in ospecs)
             print('\n# All requested packages already installed.')
-            print_packages(prefix, regex)
+            for action in action_set:
+                print_packages(action["PREFIX"], spec_regex)
         else:
             common.stdout_json_success(
                 message='All requested packages already installed.')
         return
-    elif newenv:
-        # needed in the case of creating an empty env
-        from ..instructions import LINK, UNLINK, SYMLINK_CONDA
-        if not actions[LINK] and not actions[UNLINK]:
-            actions[SYMLINK_CONDA] = [context.root_dir]
-
-    if not context.json:
-        print()
-        print("Package plan for installation in environment %s:" % prefix)
-        display_actions(actions, index, show_channel_urls=context.show_channel_urls)
-
-    if command in {'install', 'update'}:
-        check_write(command, prefix)
 
     if not context.json:
+        for actions in action_set:
+            print()
+            print("Package plan for installation in environment %s:" % actions["PREFIX"])
+            display_actions(actions, index, show_channel_urls=context.show_channel_urls)
+            # TODO: this is where the transactions should be instantiated
         common.confirm_yn(args)
+
     elif args.dry_run:
-        common.stdout_json_success(actions=actions, dry_run=True)
+        common.stdout_json_success(actions=action_set, dry_run=True)
         raise DryRunExit()
 
-    with common.json_progress_bars(json=context.json and not context.quiet):
-        try:
-            execute_actions(actions, index, verbose=not context.quiet)
-            if not (command == 'update' and args.all):
-                try:
-                    with open(join(prefix, 'conda-meta', 'history'), 'a') as f:
-                        f.write('# %s specs: %s\n' % (command, specs))
-                except IOError as e:
-                    if e.errno == errno.EACCES:
-                        log.debug("Can't write the history file")
-                    else:
-                        raise CondaIOError("Can't write the history file", e)
+    for actions in action_set:
+        if newenv:
+            # needed in the case of creating an empty env
+            from ..instructions import LINK, UNLINK, SYMLINK_CONDA
+            if not actions[LINK] and not actions[UNLINK]:
+                actions[SYMLINK_CONDA] = [context.root_prefix]
 
-        except RuntimeError as e:
-            if len(e.args) > 0 and "LOCKERROR" in e.args[0]:
-                raise LockError('Already locked: %s' % text_type(e))
-            else:
-                raise CondaRuntimeError('RuntimeError: %s' % e)
-        except SystemExit as e:
-            raise CondaSystemExit('Exiting', e)
+        if command in {'install', 'update'}:
+            check_write(command, prefix)
 
-    if newenv:
-        append_env(prefix)
-        touch_nonadmin(prefix)
-        if not context.json:
-            print(print_activate(args.name if args.name else prefix))
+        # if not context.json:
+        #     common.confirm_yn(args)
+        # elif args.dry_run:
+        #     common.stdout_json_success(actions=actions, dry_run=True)
+        #     raise DryRunExit()
 
-    if context.json:
-        common.stdout_json_success(actions=actions)
+        with common.json_progress_bars(json=context.json and not context.quiet):
+            try:
+                execute_actions(actions, index, verbose=not context.quiet)
+                if not (command == 'update' and args.all):
+                    try:
+                        with open(join(prefix, 'conda-meta', 'history'), 'a') as f:
+                            f.write('# %s specs: %s\n' % (command, ','.join(specs)))
+                    except IOError as e:
+                        if e.errno == errno.EACCES:
+                            log.debug("Can't write the history file")
+                        else:
+                            raise CondaIOError("Can't write the history file", e)
+            except SystemExit as e:
+                raise CondaSystemExit('Exiting', e)
+
+        if newenv:
+            append_env(prefix)
+            touch_nonadmin(prefix)
+            if not context.json:
+                print(print_activate(args.name if args.name else prefix))
+
+        if context.json:
+            common.stdout_json_success(actions=actions)
+
+
+def check_write(command, prefix, json=False):
+    if inroot_notwritable(prefix):
+        from .help import root_read_only
+        root_read_only(command, prefix, json=json)
+
+
+def inroot_notwritable(prefix):
+    """
+    return True if the prefix is under root and root is not writeable
+    """
+    return (abspath(prefix).startswith(context.root_prefix) and
+            not context.root_writable)
