@@ -35,7 +35,7 @@ Additional help for each command can be accessed by using:
     conda <command> -h
 """
 
-from __future__ import absolute_import, division, print_function
+from __future__ import absolute_import, division, print_function, unicode_literals
 
 import importlib
 import sys
@@ -80,29 +80,27 @@ def generate_parser():
     return p, sub_parsers
 
 
-def _main():
+def init_loggers(context=None):
+    from ..gateways.logging import initialize_logging, set_all_logger_level, set_verbosity
+    initialize_logging()
+    if context and context.json:
+        # Silence logging info to avoid interfering with JSON output
+        for logger in ('print', 'dotupdate', 'stdoutlog', 'stderrlog'):
+            getLogger(logger).setLevel(CRITICAL + 1)
+
+    if context and context.debug:
+        set_all_logger_level(DEBUG)
+    elif context and context.verbosity:
+        set_verbosity(context.verbosity)
+        log.debug("verbosity set to %s", context.verbosity)
+
+
+def _main(*args):
+    from ..base.constants import SEARCH_PATH
     from ..base.context import context
-    from ..gateways.logging import set_all_logger_level, set_verbosity
-    from ..exceptions import CommandNotFoundError
-    from ..utils import on_win
 
-    log.debug("conda.cli.main called with %s", sys.argv)
-    if len(sys.argv) > 1:
-        argv1 = sys.argv[1]
-        if argv1 in ('..activate', '..deactivate', '..checkenv', '..changeps1'):
-            import conda.cli.activate as activate
-            activate.main()
-            return
-        if argv1 in ('activate', 'deactivate'):
-
-            message = "'%s' is not a conda command.\n" % argv1
-            if not on_win:
-                message += ' Did you mean "source %s" ?\n' % ' '.join(sys.argv[1:])
-
-            raise CommandNotFoundError(argv1, message)
-
-    if len(sys.argv) == 1:
-        sys.argv.append('-h')
+    if len(args) == 1:
+        args = args + ('-h',)
 
     p, sub_parsers = generate_parser()
 
@@ -117,39 +115,68 @@ def _main():
         if "remove" in module:
             imported.configure_parser(sub_parsers, name='uninstall')
 
-    from conda.cli.find_commands import find_commands
+    from .find_commands import find_commands
 
     def completer(prefix, **kwargs):
         return [i for i in list(sub_parsers.choices) + find_commands()
                 if i.startswith(prefix)]
 
+    # when using sys.argv, first argument is generally conda or __main__.py.  Ignore it.
+    if (any(sname in args[0] for sname in ('conda', 'conda.exe', '__main__.py', 'conda-script.py'))
+        and (args[1] in list(sub_parsers.choices.keys()) + find_commands()
+             or args[1].startswith('-'))):
+        log.debug("Ignoring first argument (%s), as it is not a subcommand", args[0])
+        args = args[1:]
+
     sub_parsers.completer = completer
-    args = p.parse_args()
+    args = p.parse_args(args)
 
-    context._add_argparse_args(args)
-
-    if getattr(args, 'json', False):
-        # # Silence logging info to avoid interfering with JSON output
-        # for logger in Logger.manager.loggerDict:
-        #     if logger not in ('fetch', 'progress'):
-        #         getLogger(logger).setLevel(CRITICAL + 1)
-        for logger in ('print', 'dotupdate', 'stdoutlog', 'stderrlog'):
-            getLogger(logger).setLevel(CRITICAL + 1)
-
-    if context.debug:
-        set_all_logger_level(DEBUG)
-    elif context.verbosity:
-        set_verbosity(context.verbosity)
-        log.debug("verbosity set to %s", context.verbosity)
+    context.__init__(SEARCH_PATH, 'conda', args)
+    init_loggers(context)
 
     exit_code = args.func(args, p)
     if isinstance(exit_code, int):
         return exit_code
 
 
-def main():
+def _ensure_text_type(value):
+    # copying here from conda/common/compat.py to avoid the import
+    try:
+        return value.decode('utf-8')
+    except AttributeError:
+        # AttributeError: '<>' object has no attribute 'decode'
+        # In this case assume already text_type and do nothing
+        return value
+    except UnicodeDecodeError:
+        from requests.packages.chardet import detect
+        encoding = detect(value).get('encoding') or 'utf-8'
+        return value.decode(encoding)
+
+
+def main(*args):
+    if not args:
+        args = sys.argv
+
+    args = tuple(_ensure_text_type(s) for s in args)
+
+    log.debug("conda.cli.main called with %s", args)
+    if len(args) > 1:
+        try:
+            argv1 = args[1].strip()
+            if argv1.startswith('..'):
+                import conda.cli.activate as activate
+                activate.main()
+                return
+            elif argv1 in ('activate', 'deactivate'):
+                from ..exceptions import CommandNotFoundError
+                raise CommandNotFoundError(argv1)
+        except Exception as e:
+            from ..exceptions import handle_exception
+            init_loggers()
+            return handle_exception(e)
+
     from ..exceptions import conda_exception_handler
-    return conda_exception_handler(_main)
+    return conda_exception_handler(_main, *args)
 
 
 if __name__ == '__main__':
