@@ -3,21 +3,25 @@
 #
 # conda is distributed under the terms of the BSD 3-clause license.
 # Consult LICENSE.txt or http://opensource.org/licenses/BSD-3-Clause.
-from __future__ import absolute_import, division, print_function
+from __future__ import absolute_import, division, print_function, unicode_literals
 
-import os
-import sys
 from collections import defaultdict
+from logging import getLogger
+import os
 from os import listdir, lstat, walk
 from os.path import getsize, isdir, join
+import sys
 
 from .common import add_parser_json, add_parser_yes, confirm_yn, stdout_json
+from ..base.constants import CONDA_TARBALL_EXTENSION
 from ..base.context import context
-from ..common.disk import rm_rf
 from ..exceptions import ArgumentError
-from ..lock import LOCK_EXTENSION
+from ..gateways.disk.delete import rm_rf
+from ..gateways.disk.link import CrossPlatformStLink
 from ..utils import human_bytes
 
+log = getLogger(__name__)
+
 descr = """
 Remove unused packages and caches.
 """
@@ -72,143 +76,20 @@ def configure_parser(sub_parsers):
     p.set_defaults(func=execute)
 
 
-# work-around for python bug on Windows prior to python 3.2
-# https://bugs.python.org/issue10027
-# Adapted from the ntfsutils package, Copyright (c) 2012, the Mozilla Foundation
-class CrossPlatformStLink(object):
-    _st_nlink = None
-
-    def __call__(self, path):
-        return self.st_nlink(path)
-
-    @classmethod
-    def st_nlink(cls, path):
-        if cls._st_nlink is None:
-            cls._initialize()
-        return cls._st_nlink(path)
-
-    @classmethod
-    def _standard_st_nlink(cls, path):
-        return lstat(path).st_nlink
-
-    @classmethod
-    def _windows_st_nlink(cls, path):
-        st_nlink = cls._standard_st_nlink(path)
-        if st_nlink != 0:
-            return st_nlink
-        else:
-            # cannot trust python on Windows when st_nlink == 0
-            # get value using windows libraries to be sure of its true value
-            # Adapted from the ntfsutils package, Copyright (c) 2012, the Mozilla Foundation
-            GENERIC_READ = 0x80000000
-            FILE_SHARE_READ = 0x00000001
-            OPEN_EXISTING = 3
-            hfile = cls.CreateFile(path, GENERIC_READ, FILE_SHARE_READ, None,
-                                   OPEN_EXISTING, 0, None)
-            if hfile is None:
-                from ctypes import WinError
-                raise WinError()
-            info = cls.BY_HANDLE_FILE_INFORMATION()
-            rv = cls.GetFileInformationByHandle(hfile, info)
-            cls.CloseHandle(hfile)
-            if rv == 0:
-                from ctypes import WinError
-                raise WinError()
-            return info.nNumberOfLinks
-
-    @classmethod
-    def _initialize(cls):
-        if os.name != 'nt':
-            cls._st_nlink = cls._standard_st_nlink
-        else:
-            # http://msdn.microsoft.com/en-us/library/windows/desktop/aa363858
-            import ctypes
-            from ctypes import POINTER
-            from ctypes.wintypes import DWORD, HANDLE, BOOL
-
-            cls.CreateFile = ctypes.windll.kernel32.CreateFileW
-            cls.CreateFile.argtypes = [ctypes.c_wchar_p, DWORD, DWORD, ctypes.c_void_p,
-                                       DWORD, DWORD, HANDLE]
-            cls.CreateFile.restype = HANDLE
-
-            # http://msdn.microsoft.com/en-us/library/windows/desktop/ms724211
-            cls.CloseHandle = ctypes.windll.kernel32.CloseHandle
-            cls.CloseHandle.argtypes = [HANDLE]
-            cls.CloseHandle.restype = BOOL
-
-            class FILETIME(ctypes.Structure):
-                _fields_ = [("dwLowDateTime", DWORD),
-                            ("dwHighDateTime", DWORD)]
-
-            class BY_HANDLE_FILE_INFORMATION(ctypes.Structure):
-                _fields_ = [("dwFileAttributes", DWORD),
-                            ("ftCreationTime", FILETIME),
-                            ("ftLastAccessTime", FILETIME),
-                            ("ftLastWriteTime", FILETIME),
-                            ("dwVolumeSerialNumber", DWORD),
-                            ("nFileSizeHigh", DWORD),
-                            ("nFileSizeLow", DWORD),
-                            ("nNumberOfLinks", DWORD),
-                            ("nFileIndexHigh", DWORD),
-                            ("nFileIndexLow", DWORD)]
-            cls.BY_HANDLE_FILE_INFORMATION = BY_HANDLE_FILE_INFORMATION
-
-            # http://msdn.microsoft.com/en-us/library/windows/desktop/aa364952
-            cls.GetFileInformationByHandle = ctypes.windll.kernel32.GetFileInformationByHandle
-            cls.GetFileInformationByHandle.argtypes = [HANDLE, POINTER(BY_HANDLE_FILE_INFORMATION)]
-            cls.GetFileInformationByHandle.restype = BOOL
-
-            cls._st_nlink = cls._windows_st_nlink
-
-
-def find_lock(file_ending=LOCK_EXTENSION, extra_path=None):
-    from os.path import join
-    lock_dirs = context.pkgs_dirs[:]
-    lock_dirs += [context.root_dir]
-    for envs_dir in context.envs_dirs:
-        if os.path.exists(envs_dir):
-            for fn in os.listdir(envs_dir):
-                if os.path.isdir(join(envs_dir, fn)):
-                    lock_dirs.append(join(envs_dir, fn))
-
-    try:
-        from conda_build.config import croot
-        lock_dirs.append(croot)
-    except ImportError:
-        pass
-
-    lock_dirs = lock_dirs + list(extra_path) if extra_path else lock_dirs
-    for dir in lock_dirs:
-        if not os.path.exists(dir):
-            continue
-        for dn in os.listdir(dir):
-            if os.path.exists(join(dir, dn)) and dn.endswith(file_ending):
-                path = join(dir, dn)
-                yield path
-
-
-def rm_lock(locks, verbose=True):
-    from ..install import rm_rf
-    for path in locks:
-        if verbose:
-            print('removing: %s' % path)
-        rm_rf(path)
-
-
 def find_tarballs():
+    from ..core.package_cache import PackageCache
     pkgs_dirs = defaultdict(list)
-    for pkgs_dir in context.pkgs_dirs:
+    totalsize = 0
+    part_ext = CONDA_TARBALL_EXTENSION + '.part'
+    for package_cache in PackageCache.all_writable(context.pkgs_dirs):
+        pkgs_dir = package_cache.pkgs_dir
         if not isdir(pkgs_dir):
             continue
-        for fn in os.listdir(pkgs_dir):
-            if fn.endswith('.tar.bz2') or fn.endswith('.tar.bz2.part'):
+        root, _, filenames = next(os.walk(pkgs_dir))
+        for fn in filenames:
+            if fn.endswith(CONDA_TARBALL_EXTENSION) or fn.endswith(part_ext):
                 pkgs_dirs[pkgs_dir].append(fn)
-
-    totalsize = 0
-    for pkgs_dir in pkgs_dirs:
-        for fn in pkgs_dirs[pkgs_dir]:
-            size = getsize(join(pkgs_dir, fn))
-            totalsize += size
+                totalsize += getsize(join(root, fn))
 
     return pkgs_dirs, totalsize
 
@@ -239,20 +120,25 @@ def rm_tarballs(args, pkgs_dirs, totalsize, verbose=True):
         print(fmt % ('Total:', human_bytes(totalsize)))
         print()
 
-    if not context.json:
+    if not context.json or not context.yes:
         confirm_yn(args)
     if context.json and args.dry_run:
         return
 
     for pkgs_dir in pkgs_dirs:
         for fn in pkgs_dirs[pkgs_dir]:
-            if os.access(os.path.join(pkgs_dir, fn), os.W_OK):
+            try:
+                if rm_rf(os.path.join(pkgs_dir, fn)):
+                    if verbose:
+                        print("Removed %s" % fn)
+                else:
+                    if verbose:
+                        print("WARNING: cannot remove, file permissions: %s" % fn)
+            except (IOError, OSError) as e:
                 if verbose:
-                    print("Removing %s" % fn)
-                rm_rf(os.path.join(pkgs_dir, fn))
-            else:
-                if verbose:
-                    print("WARNING: cannot remove, file permissions: %s" % fn)
+                    print("WARNING: cannot remove, file permissions: %s\n%r" % (fn, e))
+                else:
+                    log.info("%r", e)
 
 
 def find_pkgs():
@@ -272,8 +158,6 @@ def find_pkgs():
         for pkg in pkgs:
             breakit = False
             for root, dir, files in walk(join(pkgs_dir, pkg)):
-                if breakit:
-                    break
                 for fn in files:
                     try:
                         st_nlink = cross_platform_st_nlink(join(root, fn))
@@ -284,6 +168,9 @@ def find_pkgs():
                         # print('%s is installed: %s' % (pkg, join(root, fn)))
                         breakit = True
                         break
+
+                if breakit:
+                    break
             else:
                 pkgs_dirs[pkgs_dir].append(pkg)
 
@@ -331,7 +218,7 @@ def rm_pkgs(args, pkgs_dirs, warnings, totalsize, pkgsizes,
         print(fmt % ('Total:', human_bytes(totalsize)))
         print()
 
-    if not context.json:
+    if not context.json or not context.yes:
         confirm_yn(args)
     if context.json and args.dry_run:
         return
@@ -344,27 +231,18 @@ def rm_pkgs(args, pkgs_dirs, warnings, totalsize, pkgsizes,
 
 
 def rm_index_cache():
-    from conda.install import rm_rf
-
-    rm_rf(join(context.pkgs_dirs[0], 'cache'))
+    from ..gateways.disk.delete import rm_rf
+    from ..core.package_cache import PackageCache
+    for package_cache in PackageCache.all_writable():
+        rm_rf(join(package_cache.pkgs_dir, 'cache'))
 
 
 def find_source_cache():
-    try:
-        import conda_build.source
-    except ImportError:
-        return {
-            'warnings': ["conda-build is not installed; could not clean source cache"],
-            'cache_dirs': [],
-            'cache_sizes': {},
-            'total_size': 0,
-        }
-
     cache_dirs = {
-        'source cache': conda_build.source.SRC_CACHE,
-        'git cache': conda_build.source.GIT_CACHE,
-        'hg cache': conda_build.source.HG_CACHE,
-        'svn cache': conda_build.source.SVN_CACHE,
+        'source cache': context.src_cache,
+        'git cache': context.git_cache,
+        'hg cache': context.hg_cache,
+        'svn cache': context.svn_cache,
     }
 
     sizes = {}
@@ -401,7 +279,7 @@ def rm_source_cache(args, cache_dirs, warnings, cache_sizes, total_size):
 
     print("%-40s %10s" % ("Total:", human_bytes(total_size)))
 
-    if not context.json:
+    if not context.json or not context.yes:
         confirm_yn(args)
     if context.json and args.dry_run:
         return
@@ -416,13 +294,6 @@ def execute(args, parser):
         'success': True
     }
 
-    if args.lock or args.all:
-        locks = list(find_lock())
-        json_result['lock'] = {
-            'files': locks
-        }
-        rm_lock(locks, verbose=not context.json)
-
     if args.tarballs or args.all:
         pkgs_dirs, totalsize = find_tarballs()
         first = sorted(pkgs_dirs)[0] if pkgs_dirs else ''
