@@ -3,21 +3,26 @@
 #
 # conda is distributed under the terms of the BSD 3-clause license.
 # Consult LICENSE.txt or http://opensource.org/licenses/BSD-3-Clause.
-from __future__ import absolute_import, division, print_function
+from __future__ import absolute_import, division, print_function, unicode_literals
 
+from argparse import SUPPRESS
 import collections
 import json
 import os
+from os.path import join
 import sys
+from textwrap import wrap
 
-from .common import (Completer, add_parser_json, stdout_json_success)
+from .common import Completer, add_parser_json, stdout_json_success
 from .. import CondaError
 from .._vendor.auxlib.compat import isiterable
-from .._vendor.auxlib.type_coercion import boolify
+from .._vendor.auxlib.entity import EntityEncoder
+from ..base.constants import CONDA_HOMEPAGE_URL
 from ..base.context import context
+from ..common.compat import iteritems, string_types, text_type
 from ..common.configuration import pretty_list, pretty_map
+from ..common.constants import NULL
 from ..common.yaml import yaml_dump, yaml_load
-from ..compat import iteritems, string_types
 from ..config import (rc_bool_keys, rc_list_keys, rc_other, rc_string_keys, sys_rc_path,
                       user_rc_path)
 from ..exceptions import CondaKeyError, CondaValueError, CouldntParseError
@@ -34,65 +39,31 @@ config command.  Writes to the user .condarc file (%s) by default.
 # recognize it because it removes the indentation, but at least it will be
 # valid.
 additional_descr = """
-See http://conda.pydata.org/docs/config.html for details on all the options
-that can go in .condarc.
+See `conda config --describe` or %s/docs/config.html
+for details on all the options that can go in .condarc.
 
-List keys, like
-
-  channels:
-    - conda
-    - defaults
-
-are modified with the --add and --remove options. For example
-
-    conda config --add channels r
-
-on the above configuration would prepend the key 'r', giving
-
-    channels:
-      - r
-      - conda
-      - defaults
-
-Note that the key 'channels' implicitly contains the key 'defaults' if it has
-not been configured yet.
-
-Boolean keys, like
-
-    always_yes: true
-
-are modified with --set and removed with --remove-key. For example
-
-    conda config --set always_yes false
-
-gives
-
-    always_yes: false
-
-Note that in YAML, "yes", "YES", "on", "true", "True", and "TRUE" are all
-valid ways to spell "true", and "no", "NO", "off", "false", "False", and
-"FALSE", are all valid ways to spell "false".
-
-The .condarc file is YAML, and any valid YAML syntax is allowed.
-"""
-
-
-# Note, the formatting of this is designed to work well with help2man
-example = """
 Examples:
 
-Get the channels defined in the system .condarc:
+Display all configuration values as calculated and compiled:
 
-    conda config --get channels --system
+    conda config --show
 
-Add the 'foo' Binstar channel:
+Display all identified configuration sources:
 
-    conda config --add channels foo
+    conda config --show-sources
 
-Disable the 'show_channel_urls' option:
+Describe all available configuration options:
 
-    conda config --set show_channel_urls no
-"""
+    conda config --describe
+
+Add the conda-canary channel:
+
+    conda config --add channels conda-canary
+
+Set the output verbosity to level 3 (highest):
+
+    conda config --set verbosity 3
+""" % CONDA_HOMEPAGE_URL
 
 
 class SingleValueKey(Completer):
@@ -120,7 +91,7 @@ def configure_parser(sub_parsers):
         'config',
         description=descr,
         help=descr,
-        epilog=additional_descr + example,
+        epilog=additional_descr,
     )
     add_parser_json(p)
 
@@ -133,6 +104,13 @@ def configure_parser(sub_parsers):
         config file ({user}).""".format(system=sys_rc_path,
                                         user=user_rc_path),
     )
+    location.add_argument(
+        "--env",
+        action="store_true",
+        help="Write to the active conda environment .condarc file (%s). "
+             "If no environment is active, write to the user config file (%s)."
+             "" % (os.getenv('CONDA_PREFIX', "<no active environment>"), user_rc_path),
+    )
     location.add_argument(
         "--file",
         action="store",
@@ -161,6 +139,11 @@ or the file path given by the 'CONDARC' environment variable, if it is set
         action="store_true",
         help="Validate all configuration sources.",
     )
+    action.add_argument(
+        "--describe",
+        action="store_true",
+        help="Describe available configuration parameters.",
+    )
     action.add_argument(
         "--get",
         nargs='*',
@@ -218,8 +201,8 @@ or the file path given by the 'CONDARC' environment variable, if it is set
     p.add_argument(
         "-f", "--force",
         action="store_true",
-        help="""Write to the config file using the yaml parser.  This will
-        remove any comments or structure from the file."""
+        default=NULL,
+        help=SUPPRESS,  # TODO: No longer used.  Remove in a future release.
     )
 
     p.set_defaults(func=execute)
@@ -271,41 +254,51 @@ def execute_config(args, parser):
 
     if args.show:
         from collections import OrderedDict
+
         d = OrderedDict((key, getattr(context, key))
-                        for key in sorted(('add_anaconda_token',
-                                           'add_pip_as_python_dependency',
-                                           'allow_softlinks',
-                                           'always_copy',
-                                           'always_yes',
-                                           'auto_update_conda',
-                                           'binstar_upload',
-                                           'changeps1',
-                                           'channel_alias',
-                                           'channel_priority',
-                                           'channels',
-                                           'client_tls_cert',
-                                           'client_tls_cert_key',
-                                           'create_default_packages',
-                                           'debug',
-                                           'default_channels',
-                                           'disallow',
-                                           'envs_dirs',
-                                           'json',
-                                           'offline',
-                                           'proxy_servers',
-                                           'quiet',
-                                           'shortcuts',
-                                           'show_channel_urls',
-                                           'ssl_verify',
-                                           'track_features',
-                                           'update_dependencies',
-                                           'use_pip',
-                                           'verbosity',
-                                           )))
+                        for key in context.list_parameters())
         if context.json:
-            print(json.dumps(d, sort_keys=True, indent=2, separators=(',', ': ')))
+            print(json.dumps(d, sort_keys=True, indent=2, separators=(',', ': '),
+                  cls=EntityEncoder))
         else:
+            # coerce channels
+            d['custom_channels'] = {k: text_type(v).replace(k, '')  # TODO: the replace here isn't quite right  # NOQA
+                                    for k, v in iteritems(d['custom_channels'])}
+            # TODO: custom_multichannels needs better formatting
+            d['custom_multichannels'] = {k: json.dumps([text_type(c) for c in chnls])
+                                         for k, chnls in iteritems(d['custom_multichannels'])}
+
             print('\n'.join(format_dict(d)))
+        context.validate_configuration()
+        return
+
+    if args.describe:
+        paramater_names = context.list_parameters()
+        if context.json:
+            print(json.dumps([context.describe_parameter(name) for name in paramater_names],
+                             sort_keys=True, indent=2, separators=(',', ': '),
+                             cls=EntityEncoder))
+        else:
+            for name in paramater_names:
+                details = context.describe_parameter(name)
+                aliases = details['aliases']
+                string_delimiter = details.get('string_delimiter')
+                element_types = details['element_types']
+                if details['parameter_type'] == 'primitive':
+                    print("%s (%s)" % (name, ', '.join(sorted(set(et for et in element_types)))))
+                else:
+                    print("%s (%s: %s)" % (name, details['parameter_type'],
+                                           ', '.join(sorted(set(et for et in element_types)))))
+                def_str = '  default: %s' % json.dumps(details['default_value'], indent=2,
+                                                       separators=(',', ': '),
+                                                       cls=EntityEncoder)
+                print('\n  '.join(def_str.split('\n')))
+                if aliases:
+                    print("  aliases: %s" % ', '.join(aliases))
+                if string_delimiter:
+                    print("  string delimiter: '%s'" % string_delimiter)
+                print('\n  '.join(wrap('  ' + details['description'], 70)))
+                print()
         return
 
     if args.validate:
@@ -314,6 +307,11 @@ def execute_config(args, parser):
 
     if args.system:
         rc_path = sys_rc_path
+    elif args.env:
+        if 'CONDA_PREFIX' in os.environ:
+            rc_path = join(os.environ['CONDA_PREFIX'], '.condarc')
+        else:
+            rc_path = user_rc_path
     elif args.file:
         rc_path = args.file
     else:
@@ -365,12 +363,13 @@ def execute_config(args, parser):
 
     # prepend, append, add
     for arg, prepend in zip((args.prepend, args.append), (True, False)):
+        sequence_parameters = [p for p in context.list_parameters()
+                               if context.describe_parameter(p)['parameter_type'] == 'sequence']
         for key, item in arg:
             if key == 'channels' and key not in rc_config:
                 rc_config[key] = ['defaults']
-            if key not in rc_list_keys:
-                raise CondaValueError("key must be one of %s, not %r" %
-                                      (', '.join(rc_list_keys), key))
+            if key not in sequence_parameters:
+                raise CondaValueError("Key '%s' is not a known sequence parameter." % key)
             if not isinstance(rc_config.get(key, []), list):
                 bad = rc_config[key].__class__.__name__
                 raise CouldntParseError("key %r should be a list, not %s." % (key, bad))
@@ -390,17 +389,13 @@ def execute_config(args, parser):
             arglist.insert(0 if prepend else len(arglist), item)
 
     # Set
-    set_bools, set_strings = set(rc_bool_keys), set(rc_string_keys)
     for key, item in args.set:
-        # Check key and value
-        if key in set_bools:
-            rc_config[key] = boolify(item)
-        elif key in set_strings:
-            assert isinstance(item, string_types)
-            rc_config[key] = item
-        else:
-            raise CondaValueError("Error key must be one of %s, not %s" %
-                                  (', '.join(set_bools | set_strings), key))
+        primitive_parameters = [p for p in context.list_parameters()
+                                if context.describe_parameter(p)['parameter_type'] == 'primitive']
+        if key not in primitive_parameters:
+            raise CondaValueError("Key '%s' is not a known primitive parameter." % key)
+        value = context.typify_parameter(key, item)
+        rc_config[key] = value
 
     # Remove
     for key, item in args.remove:
@@ -421,8 +416,13 @@ def execute_config(args, parser):
         del rc_config[key]
 
     # config.rc_keys
-    with open(rc_path, 'w') as rc:
-        rc.write(yaml_dump(rc_config))
+    if not args.get:
+        try:
+            with open(rc_path, 'w') as rc:
+                rc.write(yaml_dump(rc_config))
+        except (IOError, OSError) as e:
+            raise CondaError('Cannot write to condarc file at %s\n'
+                             'Caused by %r' % (rc_path, e))
 
     if context.json:
         stdout_json_success(
