@@ -4,26 +4,31 @@
 # conda is distributed under the terms of the BSD 3-clause license.
 # Consult LICENSE.txt or http://opensource.org/licenses/BSD-3-Clause.
 
-from __future__ import absolute_import, division, print_function
+from __future__ import absolute_import, division, print_function, unicode_literals
 
+from argparse import RawDescriptionHelpFormatter
 import errno
 import logging
-import sys
-from argparse import RawDescriptionHelpFormatter
 from os.path import join
+import sys
 
 from .common import (InstalledPackages, add_parser_channels, add_parser_help, add_parser_json,
                      add_parser_no_pin, add_parser_no_use_index_cache, add_parser_offline,
                      add_parser_prefix, add_parser_pscheck, add_parser_quiet,
-                     add_parser_use_index_cache, add_parser_use_local, add_parser_yes,
-                     confirm_yn, ensure_override_channels_requires_channel, ensure_use_local,
-                     names_in_specs, root_no_rm, specs_from_args, stdout_json)
-from ..api import get_index
-from ..base.context import check_write, context
-from ..common.disk import delete_trash
-from ..compat import iteritems, iterkeys
+                     add_parser_use_index_cache, add_parser_use_local, add_parser_yes, confirm_yn,
+                     create_prefix_spec_map_with_deps, ensure_override_channels_requires_channel,
+                     ensure_use_local, names_in_specs, specs_from_args, stdout_json,
+                     add_parser_insecure)
+from .install import check_write
+from ..base.constants import ROOT_NO_RM
+from ..base.context import context
+from ..common.compat import iteritems, iterkeys
+from ..common.path import is_private_env, prefix_to_env_name
 from ..console import json_progress_bars
+from ..core.index import get_index
 from ..exceptions import CondaEnvironmentError, CondaValueError, PackageNotFoundError
+from ..gateways.disk.delete import delete_trash
+from ..resolve import Resolve
 
 help = "%s a list of packages from a specified conda environment."
 descr = help + """
@@ -94,6 +99,7 @@ def configure_parser(sub_parsers, name='remove'):
     add_parser_use_local(p)
     add_parser_offline(p)
     add_parser_pscheck(p)
+    add_parser_insecure(p)
     p.add_argument(
         'package_names',
         metavar='package_name',
@@ -105,10 +111,11 @@ def configure_parser(sub_parsers, name='remove'):
 
 
 def execute(args, parser):
-    import conda.plan as plan
-    import conda.instructions as inst
-    from conda.install import linked_data
-    from conda.common.disk import rm_rf
+    from ..core.linked_data import linked_data
+    from ..gateways.disk.delete import rm_rf
+    from ..instructions import PREFIX
+    from ..plan import (add_unlink, display_actions, execute_actions, is_root_prefix,
+                        nothing_to_do, remove_actions)
 
     if not (args.all or args.package_names):
         raise CondaValueError('no package names supplied,\n'
@@ -121,41 +128,47 @@ def execute(args, parser):
     check_write('remove', prefix, json=context.json)
     ensure_use_local(args)
     ensure_override_channels_requires_channel(args)
-    channel_urls = args.channel or ()
     if not args.features and args.all:
         index = linked_data(prefix)
-        index = {dist + '.tar.bz2': info for dist, info in iteritems(index)}
+        index = {dist: info for dist, info in iteritems(index)}
     else:
-        index = get_index(channel_urls=channel_urls,
+        index = get_index(channel_urls=context.channels,
                           prepend=not args.override_channels,
                           use_local=args.use_local,
                           use_cache=args.use_index_cache,
                           prefix=prefix)
     specs = None
     if args.features:
-        features = set(args.package_names)
-        actions = plan.remove_features_actions(prefix, index, features)
-
+        specs = ['@' + f for f in set(args.package_names)]
+        actions = remove_actions(prefix, specs, index, pinned=args.pinned)
+        action_groups = actions,
     elif args.all:
-        if plan.is_root_prefix(prefix):
+        if is_root_prefix(prefix):
             raise CondaEnvironmentError('cannot remove root environment,\n'
                                         '       add -n NAME or -p PREFIX option')
-        actions = {inst.PREFIX: prefix}
-        for fkey in sorted(iterkeys(index)):
-            plan.add_unlink(actions, fkey[:-8])
-
+        actions = {PREFIX: prefix}
+        for dist in sorted(iterkeys(index)):
+            add_unlink(actions, dist)
+        action_groups = actions,
     else:
         specs = specs_from_args(args.package_names)
-        if (context.conda_in_root and plan.is_root_prefix(prefix) and
-                names_in_specs(root_no_rm, specs)):
+        r = Resolve(index)
+        prefix_spec_map = create_prefix_spec_map_with_deps(r, specs, prefix)
+
+        if (context.conda_in_root and is_root_prefix(prefix) and names_in_specs(
+                ROOT_NO_RM, specs) and not args.force):
             raise CondaEnvironmentError('cannot remove %s from root environment' %
-                                        ', '.join(root_no_rm))
-        actions = plan.remove_actions(prefix, specs, index=index,
-                                      force=args.force, pinned=args.pinned)
+                                        ', '.join(ROOT_NO_RM))
+        actions = []
+        for prfx, spcs in iteritems(prefix_spec_map):
+            index = linked_data(prfx)
+            index = {dist: info for dist, info in iteritems(index)}
+            actions.append(remove_actions(prfx, list(spcs), index=index, force=args.force,
+                                          pinned=args.pinned))
+        action_groups = tuple(actions)
 
     delete_trash()
-
-    if plan.nothing_to_do(actions):
+    if any(nothing_to_do(actions) for actions in action_groups):
         if args.all:
             print("\nRemove all packages in environment %s:\n" % prefix, file=sys.stderr)
             if not context.json:
@@ -165,42 +178,53 @@ def execute(args, parser):
             if context.json:
                 stdout_json({
                     'success': True,
-                    'actions': actions
+                    'actions': action_groups
                 })
             return
-        raise PackageNotFoundError('', 'no packages found to remove from '
-                                   'environment: %s' % prefix)
 
-    if not context.json:
-        print()
-        print("Package plan for package removal in environment %s:" % prefix)
-        plan.display_actions(actions, index)
+        pkg = str(args.package_names).replace("['", "")
+        pkg = pkg.replace("']", "")
 
-    if context.json and args.dry_run:
-        stdout_json({
-            'success': True,
-            'dry_run': True,
-            'actions': actions
-        })
-        return
+        error_message = "No packages named '%s' found to remove from environment." % pkg
+        raise PackageNotFoundError(error_message)
+
+    for action in action_groups:
+        if not context.json:
+            print()
+            print("Package plan for package removal in environment %s:" % action["PREFIX"])
+            display_actions(action, index)
+
+        if context.json and args.dry_run:
+            stdout_json({
+                'success': True,
+                'dry_run': True,
+                'actions': action_groups
+            })
+            return
 
     if not context.json:
         confirm_yn(args)
 
-    if context.json and not context.quiet:
-        with json_progress_bars():
-            plan.execute_actions(actions, index, verbose=not context.quiet)
-    else:
-        plan.execute_actions(actions, index, verbose=not context.quiet)
-        if specs:
-            try:
-                with open(join(prefix, 'conda-meta', 'history'), 'a') as f:
-                    f.write('# remove specs: %s\n' % specs)
-            except IOError as e:
-                if e.errno == errno.EACCES:
-                    log.debug("Can't write the history file")
-                else:
-                    raise
+    for actions in action_groups:
+        if context.json and not context.quiet:
+            with json_progress_bars():
+                execute_actions(actions, index, verbose=not context.quiet)
+        else:
+            execute_actions(actions, index, verbose=not context.quiet)
+            if specs:
+                try:
+                    with open(join(prefix, 'conda-meta', 'history'), 'a') as f:
+                        f.write('# remove specs: %s\n' % ','.join(specs))
+                except IOError as e:
+                    if e.errno == errno.EACCES:
+                        log.debug("Can't write the history file")
+                    else:
+                        raise
+
+        target_prefix = actions["PREFIX"]
+        if (is_private_env(prefix_to_env_name(target_prefix, context.root_prefix)) and
+                linked_data(target_prefix) == {}):
+            rm_rf(target_prefix)
 
     if args.all:
         rm_rf(prefix)
