@@ -4,20 +4,19 @@
 # conda is distributed under the terms of the BSD 3-clause license.
 # Consult LICENSE.txt or http://opensource.org/licenses/BSD-3-Clause.
 
-from __future__ import absolute_import, division, print_function
+from __future__ import absolute_import, division, print_function, unicode_literals
 
-from conda import text_type
-from conda.api import get_index
-
-from .common import Completer, Packages, add_parser_channels, add_parser_json, add_parser_known, \
-    add_parser_offline, add_parser_prefix, add_parser_use_index_cache, add_parser_use_local, \
-    disp_features, ensure_override_channels_requires_channel, ensure_use_local, stdout_json
+from .common import (Completer, Packages, add_parser_channels, add_parser_json, add_parser_known,
+                     add_parser_offline, add_parser_prefix, add_parser_use_index_cache,
+                     add_parser_use_local, disp_features, arg2spec,
+                     ensure_override_channels_requires_channel, ensure_use_local, stdout_json,
+                     add_parser_insecure)
+from ..api import get_index
 from ..base.context import context
+from ..common.compat import text_type
 from ..exceptions import CommandArgumentError, PackageNotFoundError
-from ..install import dist2quad
 from ..misc import make_icon_url
-from ..models.channel import Channel
-from ..resolve import NoPackagesFoundError, Package
+from ..resolve import MatchSpec, NoPackagesFoundError
 
 descr = """Search for packages and display their information. The input is a
 Python regular expression.  To perform a search with a search string that starts
@@ -119,17 +118,19 @@ package.""",
     add_parser_channels(p)
     add_parser_json(p)
     add_parser_use_local(p)
+    add_parser_insecure(p)
     p.set_defaults(func=execute)
 
 def execute(args, parser):
     try:
         execute_search(args, parser)
     except NoPackagesFoundError as e:
-        raise PackageNotFoundError('', text_type(e))
+        error_message = text_type(e)
+        raise PackageNotFoundError(error_message)
 
 def execute_search(args, parser):
     import re
-    from conda.resolve import Resolve
+    from ..resolve import Resolve
 
     if args.reverse_dependency:
         if not args.regex:
@@ -141,7 +142,7 @@ def execute_search(args, parser):
     ms = None
     if args.regex:
         if args.spec:
-            ms = ' '.join(args.regex.split('='))
+            ms = MatchSpec(arg2spec(args.regex))
         else:
             regex = args.regex
             if args.full_name:
@@ -156,10 +157,11 @@ def execute_search(args, parser):
 
     prefix = context.prefix_w_legacy_search
 
-    import conda.install
+    from ..core.linked_data import linked as linked_data
+    from ..core.package_cache import PackageCache
 
-    linked = conda.install.linked(prefix)
-    extracted = conda.install.extracted()
+    linked = linked_data(prefix)
+    extracted = set(pc_entry.dist.name for pc_entry in PackageCache.get_all_extracted_entries())
 
     # XXX: Make this work with more than one platform
     platform = args.platform or ''
@@ -167,8 +169,7 @@ def execute_search(args, parser):
         args.unknown = False
     ensure_use_local(args)
     ensure_override_channels_requires_channel(args, dashc=False)
-    channel_urls = args.channel or ()
-    index = get_index(channel_urls=channel_urls, prepend=not args.override_channels,
+    index = get_index(channel_urls=context.channels, prepend=not args.override_channels,
                       platform=args.platform, use_local=args.use_local,
                       use_cache=args.use_index_cache, prefix=None,
                       unknown=args.unknown)
@@ -184,42 +185,20 @@ def execute_search(args, parser):
     for name in sorted(r.groups):
         if '@' in name:
             continue
+        res = []
         if args.reverse_dependency:
-            ms_name = ms
-            for pkg in r.groups[name]:
-                for dep in r.ms_depends(pkg):
-                    if pat.search(dep.name):
-                        names.append((name, Package(pkg, r.index[pkg])))
-        else:
-            if pat and pat.search(name) is None:
-                continue
-            if ms and name != ms.split()[0]:
-                continue
-
-            if ms:
-                ms_name = ms
-            else:
-                ms_name = name
-
-            pkgs = sorted(r.get_pkgs(ms_name))
-            names.append((name, pkgs))
-
-    if args.reverse_dependency:
-        new_names = []
-        old = None
-        for name, pkg in sorted(names, key=lambda x: (x[0], x[1].name, x[1])):
-            if name == old:
-                new_names[-1][1].append(pkg)
-            else:
-                new_names.append((name, [pkg]))
-            old = name
-        names = new_names
+            res = [dist for dist in r.get_dists_for_spec(name)
+                   if any(pat.search(dep.name) for dep in r.ms_depends(dist))]
+        elif ms is not None:
+            if ms.name == name:
+                res = r.get_dists_for_spec(ms)
+        elif pat is None or pat.search(name):
+            res = r.get_dists_for_spec(name)
+        if res:
+            names.append((name, res))
 
     for name, pkgs in names:
-        if args.reverse_dependency:
-            disp_name = pkgs[0].name
-        else:
-            disp_name = name
+        disp_name = name
 
         if args.names_only and not args.outdated:
             print(name)
@@ -229,8 +208,7 @@ def execute_search(args, parser):
             json[name] = []
 
         if args.outdated:
-            vers_inst = [dist[1] for dist in map(dist2quad, linked)
-                         if dist[0] == name]
+            vers_inst = [dist.quad[1] for dist in linked if dist.quad[0] == name]
             if not vers_inst:
                 continue
             assert len(vers_inst) == 1, name
@@ -243,13 +221,13 @@ def execute_search(args, parser):
                 print(name)
                 continue
 
-        for pkg in pkgs:
-            dist = pkg.fn[:-8]
+        for dist in pkgs:
+            index_record = r.index[dist]
             if args.canonical:
                 if not context.json:
-                    print(dist)
+                    print(dist.dist_name)
                 else:
-                    json.append(dist)
+                    json.append(dist.dist_name)
                 continue
             if platform and platform != context.subdir:
                 inst = ' '
@@ -260,36 +238,38 @@ def execute_search(args, parser):
             else:
                 inst = ' '
 
+            features = r.features(dist)
+
             if not context.json:
                 print('%-25s %s  %-15s %15s  %-15s %s' % (
                     disp_name, inst,
-                    pkg.version,
-                    pkg.build,
-                    Channel(pkg.channel).canonical_name,
-                    disp_features(r.features(pkg.fn)),
+                    index_record.version,
+                    index_record.build,
+                    index_record.schannel,
+                    disp_features(features),
                 ))
                 disp_name = ''
             else:
                 data = {}
-                data.update(pkg.info)
+                data.update(index_record.dump())
                 data.update({
-                    'fn': pkg.fn,
+                    'fn': index_record.fn,
                     'installed': inst == '*',
                     'extracted': inst in '*.',
-                    'version': pkg.version,
-                    'build': pkg.build,
-                    'build_number': pkg.build_number,
-                    'channel': Channel(pkg.channel).canonical_name,
-                    'full_channel': pkg.channel,
-                    'features': list(r.features(pkg.fn)),
-                    'license': pkg.info.get('license'),
-                    'size': pkg.info.get('size'),
-                    'depends': pkg.info.get('depends'),
-                    'type': pkg.info.get('type')
+                    'version': index_record.version,
+                    'build': index_record.build,
+                    'build_number': index_record.build_number,
+                    'channel': index_record.schannel,
+                    'full_channel': index_record.channel,
+                    'features': list(features),
+                    'license': index_record.get('license'),
+                    'size': index_record.get('size'),
+                    'depends': index_record.get('depends'),
+                    'type': index_record.get('type')
                 })
 
                 if data['type'] == 'app':
-                    data['icon'] = make_icon_url(pkg.info)
+                    data['icon'] = make_icon_url(index_record.info)
                 json[name].append(data)
 
     if context.json:
