@@ -1,32 +1,32 @@
 # this module contains miscellaneous stuff which enventually could be moved
 # into other places
 
-from __future__ import print_function, division, absolute_import
+from __future__ import absolute_import, division, print_function, unicode_literals
 
+from collections import defaultdict
 import os
+from os.path import (abspath, dirname, exists, expanduser, isdir, isfile, join,
+                     relpath)
 import re
 import shutil
 import sys
-from collections import defaultdict
-from os.path import (abspath, dirname, expanduser, exists,
-                     isdir, isfile, islink, join, relpath, curdir)
 
-from .api import get_index
+from ._vendor.auxlib.path import expand
 from .base.context import context
-from .common.url import path_to_url, is_url
-from .compat import iteritems, itervalues
-from .models.channel import Channel
-from .exceptions import (CondaFileNotFoundError, ParseError, MD5MismatchError,
-                         PackageNotFoundError, CondaRuntimeError)
-from .install import (name_dist, linked as install_linked, is_fetched, is_extracted, is_linked,
-                      linked_data, find_new_location, cached_url, dist2filename)
-from .instructions import RM_FETCHED, FETCH, RM_EXTRACTED, EXTRACT, UNLINK, LINK, SYMLINK_CONDA
+from .common.compat import iteritems, iterkeys, itervalues, on_win, open
+from .common.path import url_to_path, win_path_ok
+from .common.url import is_url, join_url, path_to_url, unquote
+from .core.index import get_index, _supplement_index_with_cache
+from .core.linked_data import linked_data
+from .core.package_cache import PackageCache, ProgressiveFetchExtract
+from .exceptions import CondaFileNotFoundError, ParseError, PackageNotFoundError
+from .gateways.disk.delete import rm_rf
+from .gateways.disk.link import islink
+from .instructions import LINK, UNLINK
+from .models.dist import Dist
+from .models.index_record import IndexRecord
 from .plan import execute_actions
-from .resolve import Resolve, MatchSpec
-from .utils import md5_file, on_win
-from .common.disk import rm_rf
-from .install import LINK_COPY, LINK_HARD, LINK_SOFT
-from .install import try_hard_link
+from .resolve import MatchSpec, Resolve
 
 
 def conda_installed_files(prefix, exclude_self_build=False):
@@ -35,11 +35,10 @@ def conda_installed_files(prefix, exclude_self_build=False):
     a given prefix.
     """
     res = set()
-    for dist in install_linked(prefix):
-        meta = is_linked(prefix, dist)
+    for dist, meta in iteritems(linked_data(prefix)):
         if exclude_self_build and 'file_hash' in meta:
             continue
-        res.update(set(meta['files']))
+        res.update(set(meta.get('files', ())))
     return res
 
 
@@ -49,143 +48,66 @@ url_pat = re.compile(r'(?:(?P<url_p>.+)(?:[/\\]))?'
 def explicit(specs, prefix, verbose=False, force_extract=True, index_args=None, index=None):
     actions = defaultdict(list)
     actions['PREFIX'] = prefix
-    actions['op_order'] = RM_FETCHED, FETCH, RM_EXTRACTED, EXTRACT, UNLINK, LINK, SYMLINK_CONDA
-    linked = {name_dist(dist): dist for dist in install_linked(prefix)}
-    index_args = index_args or {}
-    index = index or {}
-    verifies = []
-    channels = set()
+
+    fetch_recs = {}
     for spec in specs:
         if spec == '@EXPLICIT':
             continue
 
-        # Format: (url|path)(:#md5)?
+        if not is_url(spec):
+            spec = unquote(path_to_url(expand(spec)))
+
+        # parse URL
         m = url_pat.match(spec)
         if m is None:
             raise ParseError('Could not parse explicit URL: %s' % spec)
-        url_p, fn, md5 = m.group('url_p'), m.group('fn'), m.group('md5')
-        if not is_url(url_p):
-            if url_p is None:
-                url_p = curdir
-            elif not isdir(url_p):
-                raise CondaFileNotFoundError(join(url_p, fn))
-            url_p = path_to_url(url_p).rstrip('/')
-        url = "{0}/{1}".format(url_p, fn)
+        url_p, fn, md5sum = m.group('url_p'), m.group('fn'), m.group('md5')
+        url = join_url(url_p, fn)
+        # url_p is everything but the tarball_basename and the md5sum
 
-        # is_local: if the tarball is stored locally (file://)
-        # is_cache: if the tarball is sitting in our cache
-        is_local = not is_url(url) or url.startswith('file://')
-        prefix = cached_url(url) if is_local else None
-        is_cache = prefix is not None
-        if is_cache:
-            # Channel information from the cache
-            schannel = 'defaults' if prefix == '' else prefix[:-2]
-        else:
-            # Channel information from the URL
-            channel, schannel = Channel(url).url_channel_wtf
-            prefix = '' if schannel == 'defaults' else schannel + '::'
+        # If the path points to a file in the package cache, we need to use
+        # the dist name that corresponds to that package. The MD5 may not
+        # match, but we will let PFE below worry about that
+        dist = None
+        if url.startswith('file:/'):
+            path = win_path_ok(url_to_path(url))
+            if dirname(path) in context.pkgs_dirs:
+                if not exists(path):
+                    raise CondaFileNotFoundError(path)
+                pc_entry = PackageCache.tarball_file_in_cache(path)
+                dist = pc_entry.dist
+                url = dist.to_url() or pc_entry.get_urls_txt_value()
+                md5sum = md5sum or pc_entry.md5sum
+        dist = dist or Dist(url)
+        fetch_recs[dist] = {'md5': md5sum, 'url': url}
 
-        fn = prefix + fn
-        dist = fn[:-8]
-        # Add explicit file to index so we'll be sure to see it later
-        if is_local:
-            index[fn] = {'fn': dist2filename(fn), 'url': url, 'md5': md5}
-            verifies.append((fn, md5))
+    # perform any necessary fetches and extractions
+    if verbose:
+        from .console import setup_verbose_handlers
+        setup_verbose_handlers()
+    link_dists = tuple(iterkeys(fetch_recs))
+    pfe = ProgressiveFetchExtract(fetch_recs, link_dists)
+    pfe.execute()
 
-        pkg_path = is_fetched(dist)
-        dir_path = is_extracted(dist)
+    # dists could have been updated with more accurate urls
+    # TODO: I'm stuck here
 
-        # Don't re-fetch unless there is an MD5 mismatch
-        # Also remove explicit tarballs from cache, unless the path *is* to the cache
-        if pkg_path and not is_cache and (is_local or md5 and md5_file(pkg_path) != md5):
-            # This removes any extracted copies as well
-            actions[RM_FETCHED].append(dist)
-            pkg_path = dir_path = None
+    # Now get the index---but the only index we need is the package cache
+    index = {}
+    _supplement_index_with_cache(index, ())
 
-        # Don't re-extract unless forced, or if we can't check the md5
-        if dir_path and (force_extract or md5 and not pkg_path):
-            actions[RM_EXTRACTED].append(dist)
-            dir_path = None
+    # unlink any installed packages with same package name
+    link_names = {index[d]['name'] for d in link_dists}
+    actions[UNLINK].extend(d for d, r in iteritems(linked_data(prefix))
+                           if r['name'] in link_names)
 
-        if not dir_path:
-            if not pkg_path:
-                pkg_path, conflict = find_new_location(dist)
-                pkg_path = join(pkg_path, dist2filename(dist))
-                if conflict:
-                    actions[RM_FETCHED].append(conflict)
-                if not is_local:
-                    if fn not in index or index[fn].get('not_fetched'):
-                        channels.add(schannel)
-                    verifies.append((dist + '.tar.bz2', md5))
-                actions[FETCH].append(dist)
-            actions[EXTRACT].append(dist)
+    # need to get the install order right, especially to install python in the prefix
+    #  before python noarch packages
+    r = Resolve(index)
+    actions[LINK].extend(link_dists)
+    actions[LINK] = r.dependency_sort({r.package_name(dist): dist for dist in actions[LINK]})
 
-        # unlink any installed package with that name
-        name = name_dist(dist)
-        if name in linked:
-            actions[UNLINK].append(linked[name])
-
-        ######################################
-        # copied from conda/plan.py   TODO: refactor
-        ######################################
-
-        # check for link action
-        fetched_dist = dir_path or pkg_path[:-8]
-        fetched_dir = dirname(fetched_dist)
-        try:
-            # Determine what kind of linking is necessary
-            if not dir_path:
-                # If not already extracted, create some dummy
-                # data to test with
-                rm_rf(fetched_dist)
-                ppath = join(fetched_dist, 'info')
-                os.makedirs(ppath)
-                index_json = join(ppath, 'index.json')
-                with open(index_json, 'w'):
-                    pass
-            if context.always_copy:
-                lt = LINK_COPY
-            elif try_hard_link(fetched_dir, prefix, dist):
-                lt = LINK_HARD
-            elif context.allow_softlinks and not on_win:
-                lt = LINK_SOFT
-            else:
-                lt = LINK_COPY
-            actions[LINK].append('%s %d' % (dist, lt))
-        except (OSError, IOError):
-            actions[LINK].append('%s %d' % (dist, LINK_COPY))
-        finally:
-            if not dir_path:
-                # Remove the dummy data
-                try:
-                    rm_rf(fetched_dist)
-                except (OSError, IOError):
-                    pass
-
-    ######################################
-    # ^^^^^^^^^^ copied from conda/plan.py
-    ######################################
-
-    # Pull the repodata for channels we are using
-    if channels:
-        index_args = index_args or {}
-        index_args = index_args.copy()
-        index_args['prepend'] = False
-        index_args['channel_urls'] = list(channels)
-        index.update(get_index(**index_args))
-
-    # Finish the MD5 verification
-    for fn, md5 in verifies:
-        info = index.get(fn)
-        if info is None:
-            raise PackageNotFoundError(fn, "no package '%s' in index" % fn)
-        if md5 and 'md5' not in info:
-            sys.stderr.write('Warning: cannot lookup MD5 of: %s' % fn)
-        if md5 and info['md5'] != md5:
-            raise MD5MismatchError('MD5 mismatch for: %s\n   spec: %s\n   repo: %s'
-                                   % (fn, md5, info['md5']))
-
-    execute_actions(actions, index=index, verbose=verbose)
+    execute_actions(actions, index, verbose=verbose)
     return actions
 
 
@@ -262,7 +184,7 @@ def touch_nonadmin(prefix):
     """
     Creates $PREFIX/.nonadmin if sys.prefix/.nonadmin exists (on Windows)
     """
-    if on_win and exists(join(context.root_dir, '.nonadmin')):
+    if on_win and exists(join(context.root_prefix, '.nonadmin')):
         if not isdir(prefix):
             os.makedirs(prefix)
         with open(join(prefix, '.nonadmin'), 'w') as fo:
@@ -286,7 +208,7 @@ def clone_env(prefix1, prefix2, verbose=True, quiet=False, index_args=None):
     """
     untracked_files = untracked(prefix1)
 
-    # Discard conda and any package that depends on it
+    # Discard conda, conda-env and any package that depends on them
     drecs = linked_data(prefix1)
     filter = {}
     found = True
@@ -300,15 +222,21 @@ def clone_env(prefix1, prefix2, verbose=True, quiet=False, index_args=None):
                 filter['conda'] = dist
                 found = True
                 break
+            if name == "conda-env":
+                filter["conda-env"] = dist
+                found = True
+                break
             for dep in info.get('depends', []):
                 if MatchSpec(dep).name in filter:
                     filter[name] = dist
                     found = True
+
     if filter:
         if not quiet:
-            print('The following packages cannot be cloned out of the root environment:')
+            fh = sys.stderr if context.json else sys.stdout
+            print('The following packages cannot be cloned out of the root environment:', file=fh)
             for pkg in itervalues(filter):
-                print(' - ' + pkg)
+                print(' - ' + pkg.dist_name, file=fh)
             drecs = {dist: info for dist, info in iteritems(drecs) if info['name'] not in filter}
 
     # Resolve URLs for packages that do not have URLs
@@ -321,34 +249,31 @@ def clone_env(prefix1, prefix2, verbose=True, quiet=False, index_args=None):
         index = get_index(**index_args)
         r = Resolve(index, sort=True)
         for dist in unknowns:
-            name = name_dist(dist)
-            fn = dist2filename(dist)
+            name = dist.dist_name
+            fn = dist.to_filename()
             fkeys = [d for d in r.index.keys() if r.index[d]['fn'] == fn]
             if fkeys:
                 del drecs[dist]
-                dist = sorted(fkeys, key=r.version_key, reverse=True)[0]
-                drecs[dist] = r.index[dist]
+                dist_str = sorted(fkeys, key=r.version_key, reverse=True)[0]
+                drecs[Dist(dist_str)] = r.index[dist_str]
             else:
                 notfound.append(fn)
     if notfound:
         what = "Package%s " % ('' if len(notfound) == 1 else 's')
-        notfound = '\n'.join(' - ' + fn for fn in notfound)
-        msg = '%s missing in current %s channels:%s' % (what, context.subdir, notfound)
-        raise CondaRuntimeError(msg)
+        raise PackageNotFoundError(what)
 
     # Assemble the URL and channel list
     urls = {}
     for dist, info in iteritems(drecs):
-        fkey = dist + '.tar.bz2'
+        fkey = dist
         if fkey not in index:
-            info['not_fetched'] = True
-            index[fkey] = info
+            index[fkey] = IndexRecord.from_objects(info, not_fetched=True)
             r = None
         urls[dist] = info['url']
 
     if r is None:
         r = Resolve(index)
-    dists = r.dependency_sort(urls.keys())
+    dists = r.dependency_sort({d.quad[0]: d for d in urls.keys()})
     urls = [urls[d] for d in dists]
 
     if verbose:
@@ -393,9 +318,6 @@ def make_icon_url(info):
     if info.get('channel') and info.get('icon'):
         base_url = dirname(info['channel'])
         icon_fn = info['icon']
-        # icon_cache_path = join(pkgs_dir, 'cache', icon_fn)
-        # if isfile(icon_cache_path):
-        #    return url_path(icon_cache_path)
         return '%s/icons/%s' % (base_url, icon_fn)
     return ''
 
@@ -413,4 +335,4 @@ def list_prefixes():
                 prefix = join(envs_dir, dn)
                 yield prefix
 
-    yield context.root_dir
+    yield context.root_prefix
