@@ -3,12 +3,13 @@ from __future__ import absolute_import, division, print_function, unicode_litera
 
 from itertools import chain
 from logging import getLogger
-from requests.packages.urllib3.util import Url
 
-from ..base.constants import DEFAULT_CHANNELS_UNIX, DEFAULT_CHANNELS_WIN, UTF8
+from ..base.constants import (DEFAULTS_CHANNEL_NAME, DEFAULT_CHANNELS_UNIX, DEFAULT_CHANNELS_WIN,
+                              MAX_CHANNEL_PRIORITY, UNKNOWN_CHANNEL)
 from ..base.context import context
-from ..common.compat import iteritems, odict, with_metaclass
-from ..common.url import (has_scheme, is_url, is_windows_path, join_url, on_win, path_to_url,
+from ..common.compat import ensure_text_type, isiterable, iteritems, odict, with_metaclass
+from ..common.path import is_path, win_path_backout
+from ..common.url import (Url, has_scheme, is_url, join_url, path_to_url,
                           split_conda_url_easy_parts, split_scheme_auth_token, urlparse)
 
 try:
@@ -26,14 +27,6 @@ def get_conda_build_local_url():
     return context.local_build_root,
 
 
-"""
-scheme <> auth <> location <> token <> channel <> subchannel <> platform <> package_filename
-
-channel <> subchannel <> namespace <> package_name
-
-"""
-
-
 def tokenized_startswith(test_iterable, startswith_iterable):
     return all(t == sw for t, sw in zip(test_iterable, startswith_iterable))
 
@@ -163,8 +156,7 @@ class ChannelType(type):
             elif value in Channel._cache_:
                 return Channel._cache_[value]
             else:
-                c = Channel.from_value(value)
-                Channel._cache_[value] = c
+                c = Channel._cache_[value] = Channel.from_value(value)
                 return c
         else:
             return super(ChannelType, cls).__call__(*args, **kwargs)
@@ -172,6 +164,14 @@ class ChannelType(type):
 
 @with_metaclass(ChannelType)
 class Channel(object):
+    """
+    Channel:
+    scheme <> auth <> location <> token <> channel <> subchannel <> platform <> package_filename
+
+    Package Spec:
+    channel <> subchannel <> namespace <> package_name
+
+    """
     _cache_ = dict()
 
     @staticmethod
@@ -206,19 +206,18 @@ class Channel(object):
 
     @staticmethod
     def from_value(value):
-        if value is None:
-            return Channel(name="<unknown>")
-        if hasattr(value, 'decode'):
-            value = value.decode(UTF8)
+        if value in (None, '<unknown>', 'None:///<unknown>', 'None'):
+            return Channel(name=UNKNOWN_CHANNEL)
+        value = ensure_text_type(value)
         if has_scheme(value):
-            if value.startswith('file:') and on_win:
-                value = value.replace('\\', '/')
+            if value.startswith('file:'):
+                value = win_path_backout(value)
             return Channel.from_url(value)
-        elif value.startswith(('./', '..', '~', '/')) or is_windows_path(value):
+        elif is_path(value):
             return Channel.from_url(path_to_url(value))
         elif value.endswith('.tar.bz2'):
-            if value.startswith('file:') and on_win:
-                value = value.replace('\\', '/')
+            if value.startswith('file:'):
+                value = win_path_backout(value)
             return Channel.from_url(value)
         else:
             # at this point assume we don't have a bare (non-scheme) url
@@ -264,9 +263,20 @@ class Channel(object):
 
         # fall back to the equivalent of self.base_url
         # re-defining here because base_url for MultiChannel is None
-        return "%s://%s" % (self.scheme, join_url(self.location, self.name))
+        if self.scheme:
+            return "%s://%s" % (self.scheme, join_url(self.location, self.name))
+        else:
+            return join_url(self.location, self.name).lstrip('/')
+
+    def urls(self, with_credentials=False, subdirs=None):
+        if subdirs is None:
+            subdirs = context.subdirs
+
+        assert isiterable(subdirs), subdirs  # subdirs must be a non-string iterable
+
+        if self.canonical_name == UNKNOWN_CHANNEL:
+            return Channel(DEFAULTS_CHANNEL_NAME).urls(with_credentials, subdirs)
 
-    def urls(self, with_credentials=False, platform=None):
         base = [self.location]
         if with_credentials and self.token:
             base.extend(['t', self.token])
@@ -274,8 +284,14 @@ class Channel(object):
         base = join_url(*base)
 
         def _platforms():
-            p = platform or self.platform or context.subdir
-            return (p, 'noarch') if p != 'noarch' else ('noarch',)
+            if self.platform:
+                yield self.platform
+                if self.platform != 'noarch':
+                    yield 'noarch'
+            else:
+                for subdir in subdirs:
+                    yield subdir
+
         bases = (join_url(base, p) for p in _platforms())
 
         if with_credentials and self.auth:
@@ -284,6 +300,9 @@ class Channel(object):
             return ["%s://%s" % (self.scheme, b) for b in bases]
 
     def url(self, with_credentials=False):
+        if self.canonical_name == UNKNOWN_CHANNEL:
+            return None
+
         base = [self.location]
         if with_credentials and self.token:
             base.extend(['t', self.token])
@@ -293,7 +312,8 @@ class Channel(object):
             if self.package_filename:
                 base.append(self.package_filename)
         else:
-            base.append(context.subdir)
+            first_non_noarch = next((s for s in context.subdirs if s != 'noarch'), 'noarch')
+            base.append(first_non_noarch)
 
         base = join_url(*base)
 
@@ -304,14 +324,16 @@ class Channel(object):
 
     @property
     def base_url(self):
+        if self.canonical_name == UNKNOWN_CHANNEL:
+            return None
         return "%s://%s" % (self.scheme, join_url(self.location, self.name))
 
     def __str__(self):
-        return self.base_url
+        return self.base_url or ""
 
     def __repr__(self):
-        return ("Channel(scheme=%s, auth=%s, location=%s, token=%s, name=%s, platform=%s, "
-                "package_filename=%s)" % (self.scheme,
+        return ("Channel(scheme=%r, auth=%r, location=%r, token=%r, name=%r, platform=%r, "
+                "package_filename=%r)" % (self.scheme,
                                           self.auth and "%s:<PASSWORD>" % self.auth.split(':')[0],
                                           self.location,
                                           self.token and "<TOKEN>",
@@ -334,6 +356,9 @@ class Channel(object):
     def __bool__(self):
         return self.__nonzero__()
 
+    def __json__(self):
+        return self.__dict__
+
     @property
     def url_channel_wtf(self):
         return self.base_url, self.canonical_name
@@ -360,15 +385,16 @@ class MultiChannel(Channel):
     def canonical_name(self):
         return self.name
 
-    def urls(self, with_credentials=False, platform=None):
-        if platform and platform != context.subdir and self.name == 'defaults':
-            # necessary shenanigan because different platforms have different default channels
-            urls = DEFAULT_CHANNELS_WIN if 'win' in platform else DEFAULT_CHANNELS_UNIX
-            ca = context.channel_alias
-            _channels = tuple(Channel.make_simple_channel(ca, v) for v in urls)
-        else:
-            _channels = self._channels
-        return list(chain.from_iterable(c.urls(with_credentials, platform) for c in _channels))
+    def urls(self, with_credentials=False, subdirs=None):
+        _channels = self._channels
+        if self.name == 'defaults':
+            platform = next((s for s in reversed(subdirs or context.subdirs) if s != 'noarch'), '')
+            if platform != context.subdir:
+                # necessary shenanigan because different platforms have different default channels
+                urls = DEFAULT_CHANNELS_WIN if 'win' in platform else DEFAULT_CHANNELS_UNIX
+                ca = context.channel_alias
+                _channels = tuple(Channel.make_simple_channel(ca, v) for v in urls)
+        return list(chain.from_iterable(c.urls(with_credentials, subdirs) for c in _channels))
 
     @property
     def base_url(self):
@@ -378,17 +404,23 @@ class MultiChannel(Channel):
         return None
 
 
-def prioritize_channels(channels, with_credentials=True, platform=None):
+def prioritize_channels(channels, with_credentials=True, subdirs=None):
+    # prioritize_channels returns and OrderedDict with platform-specific channel
+    #   urls as the key, and a tuple of canonical channel name and channel priority
+    #   number as the value
     # ('https://conda.anaconda.org/conda-forge/osx-64/', ('conda-forge', 1))
     result = odict()
-    for q, chn in enumerate(channels):
+    for channel_priority, chn in enumerate(channels):
         channel = Channel(chn)
-        for url in channel.urls(with_credentials, platform):
+        for url in channel.urls(with_credentials, subdirs):
             if url in result:
                 continue
-            result[url] = channel.canonical_name, q
+            result[url] = channel.canonical_name, min(channel_priority, MAX_CHANNEL_PRIORITY - 1)
     return result
 
 
 def offline_keep(url):
     return not context.offline or not is_url(url) or url.startswith('file:/')
+
+
+context.register_reset_callaback(Channel._reset_state)
